#!/usr/bin/env python
# coding: utf-8
'''Sites information de Iydon.
'''
__all__ = ('get_all_links', 'get_link_by_subdomain')
def __dir__() -> list: return __all__


from .config import domain, subdomains
from .utils import typeassert, domain_join


@typeassert(str, bool, bool)
def get_link_by_subdomain(subdomain, parse=False, full=False) -> str:
	'''Get link by subdomain.

	Argument
	=======
		subdomain: str
		parse: bool, whether to parse
		full: bool, whether to return full link

	Return
	=======
		str, full link
	'''
	assert subdomain in subdomains, '`{}` not in subdomains.'.format(subdomain)
	url = (subdomains[subdomain].link, ) if parse else (subdomain, domain)
	return domain_join(url, full, subdomains[subdomain].https)


@typeassert(bool, bool)
def get_all_links(parse=False, full=False) -> list:
	'''Get all links de Iydon.

	Argument
	=======
		parse: bool, whether to parse
		full: bool, whether to return full link

	Return
	=======
		list with all str, full links
	'''
	return [get_link_by_subdomain(subdomain, parse, full) \
		for subdomain in subdomains]



#!/usr/bin/env python
# coding: utf-8
'''Deployment de Iydon.
'''
__all__ = ('Python_packages', 'Overleaf', 'LaTeX', 'R')
def __dir__() -> list: return __all__


import pip
if hasattr(pip, 'main'):
	from pip import main as pip_
else:
	from pip._internal import main as pip_

from . import config
from .config import py_pkgs_src, py_pkgs
from .config import overleaf_git, overleaf_man
from .config import latex_git
from .config import r_git

from .sites import get_link_by_subdomain
from .utils import typeassert, doc_format, yes_or_no


config.overleaf_man = overleaf_man = \
	get_link_by_subdomain('mma', full=True) + overleaf_man


@doc_format(set(py_pkgs.keys()))
@typeassert(bool, bool, bool, str)
def Python_packages(ask=True, all_=False, user=False, source='', **kwargs) -> bool:
	'''Install Python packages.

	Argument
	=======
		ask: bool, whether to use `yes_or_no`
		all_: bool, whether to install all packages
		user: bool, whether to use `--user`
		source: str, which PyPi website to use
		kwargs: in {}
	'''
	options = ['-i', source or py_pkgs_src]
	if user: options+=['--user']
	def install(pkgs:list):
		return pip_(['install'] + pkgs + options)

	_locals = dict()
	result = 0

	for pkg in py_pkgs:
		_locals[pkg] = kwargs.get(pkg, None)
		if _locals[pkg] is not None:
			continue

		if ask:
			msg = pkg.replace('_', ' ').capitalize()
			_locals[pkg] = yes_or_no(msg=msg, empty=False)
		elif all_:
			_locals[pkg] = True

	for pkg in py_pkgs:
		if _locals[pkg]:
			result += install(py_pkgs[pkg])

	return result==0


@typeassert()
def Overleaf() -> dict:
	'''Manual for deploying Overleaf.
	'''
	return {
		'github': overleaf_git,
		'manual': config.overleaf_man
	}


@typeassert()
def LaTeX() -> dict:
	'''Manual for deploying TeX live.
	'''
	return {
		'github': latex_git
	}


@typeassert()
def R() -> dict:
	'''Manual for deploying R.
	'''
	return {
		'github': r_git
	}



#!/usr/bin/env python
# coding: utf-8
'''Configuration file de Iydon.
'''
__all__ = ('embed_colors', 'py_pkgs_src', 'py_pkgs', 'overleaf_git', 'overleaf_man', 'name_en', \
	'name_fr', 'name_zh', 'emails', 'github', 'linkedin', 'domain', 'subdomains')
def __dir__() -> list: return __all__


from .models import ParseRecord


# __main__
embed_colors = 'Linux'


# deploy
py_pkgs_src = 'https://pypi.mirrors.ustc.edu.cn/simple'
py_pkgs = {
	'basic': ['bs4', 'faker', 'ipython', 'jupyter', 'tqdm'],
	'flask': ['alembic', 'bleach', 'blinker', 'click', 'dominate', 'flask', \
		'bootstrap-flask', 'flask-login', 'flask-mail', 'flask-migrate', 'flask-moment', \
		'flask-pagedown', 'flask-sqlalchemy', 'flask-wtf', 'html5lib', 'itsdangerous', 'jinja2', \
		'mako', 'markdown', 'markupsafe', 'python-dateutil', 'python-editor', 'six', 'sqlalchemy', \
		'visitor', 'webencodings', 'werkzeug', 'wtforms'],
	'research': ['numpy', 'scipy', 'pandas', 'sympy', 'matplotlib', 'tushare', 'cython'],
	'digit_image_processing': ['pillow'],
	'natural_language_processing': ['nltk', 'jieba'],
	'machine_learning': ['sklearn'],
	'deep_learning': ['torch', 'torchvision']
}

overleaf_git = 'https://github.com/overleaf/overleaf'
overleaf_man = '/2018/10/ShareLaTeX-LaTeX/'

latex_git = 'https://github.com/OsbertWang/install-latex'

r_git = 'https://github.com/Vianlee/R'


# info
name_en = 'Iydon Liang'
name_fr = 'Iydon Svégio Liang'
name_zh = '梁钰栋'
emails = ('11711217@mail.sustech.edu.cn', 'liangiydon@gmail.com')
github = 'https://github.com/Iydon'
linkedin = 'https://www.linkedin.com/in/钰栋-梁-0069ab185/'
cv = '/cv'


# sites
domain = 'svegio.top'
subdomains = {
	'math': ParseRecord('10.20.6.187', 'A', False),
	'mma':  ParseRecord('sustech-mma.github.io', 'CNAME', True),
	'wic':  ParseRecord('mic-sd.github.io', 'CNAME', True),
	'www':  ParseRecord('iydon.github.io', 'CNAME', True)
}



#!/usr/bin/env python
# coding: utf-8
'''*Just when you thought, Python could not be more fun.*
'''
__all__ = ('references', 'hello_world', 'zen', 'antigravity_comics', 'braces', 'cool_identifier', 'FLUFL', 'InPynite',
	'not_for_the_faint_of_heart')
def __dir__() -> list: return __all__


from .models import DemoCode


references = ('https://github.com/greyli/shedan',
	'https://github.com/OrkoHunter/python-easter-eggs')

hello_world = DemoCode(
	'import __hello__',
	'- 在 Python 中打印「Hello, World!」最简单的方式。')

zen = DemoCode(
	'import this',
	'- Python 设计哲学与代码风格指南。\n' \
	'- 《Python 之禅》在 PEP 20 中引入，它本应该是 20 条格言，' \
	'但是最终只写了 19 条，也许只是为了说明在文件的结尾总是应该保留一个空行。')

antigravity_comics = DemoCode(
	'import antigravity',
	'- 这会打开这个 xkcd 漫画，漫画展示了 Python 有多么简单易学。')

braces = DemoCode(
	'from __future__ import braces',
	'- 这用来快速结束任何关于在 Python 中引入大括号的讨论——不可能！')

cool_identifier = DemoCode(
	'from math import pi\n' \
	'π = pi',
	'- Python 3 中支持使用 Unicode 字符作为变量名。尽管如此，使用非英文字符作为变量名' \
	'可能不是一个好主意，不过它确实会让处理科学公式的工作变得更有意思。')

FLUFL = DemoCode(
	'from __future__ import barry_as_FLUFL\n' \
	'1 <> 2',
	'- 在认识到 Python 3.0 的不等运算符（!=）非常糟糕，是手指疼痛导致的错误后，' \
	'FLUFL（指 Uncle Barry）重新恢复钻石型操作符（<>）作为唯一的不等运算符拼写。\n' \
	'- PEP 401 是一个愚人节玩笑（从编号可以看出来）。这个 PEP 声明 Guido van Rossum 要退休了。他会获得一个新的头衔，' \
	'叫做「BDEVIL」（Benevolent Dictator Emeritus Vacationing Indefinitely from the Language，' \
	'去度无限期语言假期的仁慈退休独裁者），接任者将会是 Barry Warsaw（即 Uncle Barry），' \
	'Uncle Barry 的官方头衔是「FLUFL」（Friendly Language Uncle For Life，终生友好语言叔叔）。')

InPynite = DemoCode(
	'infinity = float("infinity")\n' \
	'print(hash(infinity))',
	'- 在 Python3 中，`hash(float("-inf"))` 将会生成 -10^5 x π，而在 Python 2 中则是 -271828（即 10^5 x e）。')

not_for_the_faint_of_heart = DemoCode(
	'import types\n' \
	'help(types.CodeType)',
	'- 如果你开始钻进 Python 的内部，你会在 help 输出看到一个关于 `types.CodeType` 的警告：不适合心脏脆弱者。')



#!/usr/bin/env python
# coding: utf-8
'''Set up package Iydon.
'''
import setuptools


with open("README.md", "r") as f:
	long_description = f.read()

setuptools.setup(
	name="iydon",
	version="0.5.1",
	author="Iydon Liang",
	author_email="11711217@mail.sustech.edu.cn",
	license='MIT License',
	description="personal api",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/iydon/iydon",
	packages=setuptools.find_packages(),
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	python_requires='>=3.6',
	install_requires=[
		'IPython',
		'pip',
		'requests'
	],
)



#!/usr/bin/env python
# coding: utf-8
'''Auxiliary models de Iydon.
'''
__all__ = ('ParseRecord', )
def __dir__() -> list: return __all__


from collections import namedtuple


ParseRecord = namedtuple('ParseRecord', ('link', 'type', 'https'))


class DemoCode:
	'''Demo code with some notes.
	'''
	__slots__ = ('_code', '_note')

	def __init__(self, code, note):
		self._code = code
		self._note = note

	def __repr__(self):
		print(self._note)
		return self._code

	@property
	def code(self): return self._code

	@property
	def note(self): return self._note

	def run(self, debug=False):
		'''Execute the demo code.

		Return
		=======
			None or Boolean (if debug)
		'''
		flag = True
		try:
			if isinstance(self.code, (tuple, list)):
				for code in self.code:
					exec(code)
			elif isinstance(self.code, str):
				exec(self.code)
		except Exception as e:
			print(e)
			flag = False

		if debug: return flag



#!/usr/bin/env python
# coding: utf-8
'''Auxiliary file de Iydon.
'''
__all__ = ('typeassert', 'domain_join', 'yes_or_no', 'json_loads_online', 'ping_link')
def __dir__() -> list: return __all__


from inspect import signature
from functools import wraps
from json import loads
from requests import get, head
from typing import Iterable


def typeassert(*t_args, **t_kwargs):
	'''Enforce type check on function using decorator.

	Todo
	=======
	Check the type of return value.

	References
	=======
	(Python Cookbook)[https://python3-cookbook.readthedocs.io/zh_CN/latest/c09/p07_enforcing_type_check_on_function_using_decorator.html]
	'''
	def decorate(func):
		# If in optimized mode, disable type checking
		if not __debug__:
			return func

		# Map function argument names to supplied types
		σ = signature(func)
		bound_types = σ.bind_partial(*t_args, **t_kwargs).arguments

		@wraps(func)
		def wrapper(*args, **kwargs):
			bound_values = σ.bind(*args, **kwargs)
			# Enforce type assertions across supplied arguments
			for name, value in bound_values.arguments.items():
				if name in bound_types:
					if not isinstance(value, bound_types[name]):
						raise TypeError(
							'Argument `{}` must be {}.'.format(name, bound_types[name])
						)
			return func(*args, **kwargs)
		return wrapper
	return decorate


@typeassert()
def doc_format(*args, **kwargs):
	'''__doc__.format
	'''
	def decorate(func):
		func.__doc__ = func.__doc__.format(*args, **kwargs)
		return func
	return decorate


@typeassert(Iterable, bool, bool)
def domain_join(domains, full=False, https=False):
	'''Join domain and subdomain.

	Argument
	=======
		domains: Iterable[str]
		full: bool, whether to return full link
		https: bool, whether to use HTTP Secure
	'''
	# If in optimized mode, disable type checking
	if __debug__:
		for value in domains:
			assert isinstance(value, str), 'Argument must be Iterable with all str.'

	if full:
		return ('https://' if https else 'http://') + '.'.join(domains)
	else:
		return '.'.join(domains)


@typeassert(str, bool)
def yes_or_no(msg='', empty=True):
	'''Yes or No.

	Argument
	=======
		msg: str, prompt message
		empth: bool, if input is empty, then return `empty`
	'''
	prompt = ' [y/N, empty is {}]: '.format('y' if empty else 'N')
	while True:
		option = input(msg + prompt)
		if option:
			if 'y' in option.lower():
				return True
			elif 'n' in option.lower():
				return False
		else:
			return empty


@typeassert(str)
def json_loads_online(url):
	'''Deserialize json data from `url`.

	Todo
	=======
	[GitHub API](https://api.github.com/)

	Argument
	=======
		url: str
	'''
	response = get(url)
	return loads(response.text)


@typeassert(str, (int, float))
def ping_link(link, timeout=3, **kwargs):
	'''Ping link from `link`.

	Argument
	=======
		link: str
		timeout: int or float, timeout seconds
		kwargs: keywords argument of `requests.request`
	'''
	try:
		response = head(link,  timeout=timeout, **kwargs)
		return response.status_code < 400 \
			or response.status_code in (999, )
	except Exception as e:
		print(e)
		return False
